#ifndef NO_DYNAMIC_MAPS

#include <sys_defs.h>
#include <string.h>
#include <stdlib.h>
#include <dlfcn.h>

#include <msg.h>
#include <vstring.h>
#include "postfix_plugin.h"

#include "../global/mail_params.h"

static void *
plugin_load(VSTRING **libname, const char *lib_prefix, const char *lib_type,
	    int warn)
{
    const char *myname = "plugin_load";
    void *handle;

    *libname = vstring_alloc(64);
    vstring_sprintf(*libname, "%s/%s_%s.so", var_plugin_dir,
		    lib_prefix, lib_type);

    if ((handle = dlopen(vstring_str(*libname), RTLD_NOW)) == 0
	&& (warn || msg_verbose))
	msg_warn("%s: error loading %s: %s", myname,
		 vstring_str(*libname), dlerror());

    return handle;
}

static void *
plugin_lookup(void *handle, VSTRING *libname, const char *sym_prefix,
	      const char *sym_type, const char *sym_suffix, int warn)
{
    const char *myname = "plugin_lookup";
    VSTRING *funcname;
    void *sym;
    const char *dlerrmsg;

    funcname = vstring_alloc(32);
    vstring_sprintf(funcname, "%s_%s_%s", sym_prefix, sym_type, sym_suffix);

    sym = dlsym(handle, vstring_str(funcname));
    if ((dlerrmsg = dlerror()) != 0 || (sym == 0))
    {
	if (warn || msg_verbose)
	    msg_warn("%s: symbol \"%s\" not found in %s: %s",
		     myname, vstring_str(funcname), vstring_str(libname),
		     (dlerrmsg ?: "null symbol address"));
	sym = 0;
    }
    vstring_free(funcname);
    return sym;
}

void *
postfix_load_symbol(const char *prefix, const char *type, const char *suffix,
		    int warn)
{
	VSTRING *libname;
	void *handle;
	void *sym = 0;

	if ((handle = plugin_load(&libname, prefix, type, warn)))
	    sym = plugin_lookup(handle, libname, prefix, type, suffix, warn);

	vstring_free(libname);
	return sym;
}

#endif /* ! NO_DYNAMIC_MAPS */
